local addonName, a = ...
local L = a.Localize
local s = SpellFlashAddon
local c = BittensGlobalTables.GetTable("BittensSpellFlashLibrary")
local u = BittensGlobalTables.GetTable("BittensUtilities")

local GetTime = GetTime
local SPELL_POWER_SHADOW_ORBS = SPELL_POWER_SHADOW_ORBS
local UnitSpellHaste = UnitSpellHaste
local min = math.min
local max = math.max
local string = string

a.Rotations = {}

local function monitorMending(spellID, target)
   if spellID == c.GetID("Prayer of Mending Buff") then
      a.PrayerOfMendingTarget = target
      c.Debug("Event", "Prayer of Mending target:", a.PrayerOfMendingTarget)
   end
end

-------------------------------------------------------------------- Discipline
a.Rotations.Discipline = {
   Spec = 1,

   UsefulStats = { "Intellect", "Spirit", "Crit", "Haste", "Multistrike" },

   FlashInCombat = function()
      c.FlashAll(
         "Archangel",
         "Prayer of Mending for Discipline",
         "Flash Heal under Surge of Light",
         "Mindbender for Mana",
         "Desperate Prayer",
         "Dispel Magic",
         "Silence"
      )
   end,

   FlashAlways = function()
      c.FlashAll(
         "Power Word: Fortitude"
      )
   end,

   AuraApplied = monitorMending,
}

-------------------------------------------------------------------------- Holy
a.Rotations.Holy = {
   Spec = 2,

   UsefulStats = { "Multistrike", "Haste", "Intellect", "Spirit", "Crit" },

   FlashInCombat = function()
      c.FlashAll(
         "Lightwell",
         "Prayer of Mending for Holy",
         "Heal under Serendipity",
         "Prayer of Healing under Serendipity",
         "Flash Heal under Surge of Light",
         "Binding Heal",
         "Mindbender for Mana",
         "Desperate Prayer",
         "Dispel Magic"
      )
   end,

   FlashAlways = function()
      c.FlashAll(
         "Power Word: Fortitude",
         "Chakra"
      )
   end,

   CastSucceeded = monitorMending,
}

------------------------------------------------------------------------ Shadow
local lastSWDCast = 0
local lastSWDCD = 0
local swdGivesOrb = true

function a.swdGivesOrb()
   if c.HasSpell("Enhanced Shadow Word: Death") then
      return true
   else
      return swdGivesOrb
   end
end

a.FlayTick = 1
a.Rotations.Shadow = {
   Spec = 3,

   UsefulStats = {
      "Intellect", "Spell Hit", "Hit from Spirit", "Crit", "Haste"
   },

   PreFlash = function()
       a.maxOrbs = c.HasSpell("Enhanced Shadow Orbs") and 5 or 3
   end,

   FlashInCombat = function()
      a.SinceSWD = GetTime() - lastSWDCast
      local swdcd = c.GetCooldown("Shadow Word: Death", false, 8)
      if a.SinceSWD < 9 and swdcd < lastSWDCD then
         swdGivesOrb = false
      else
         swdGivesOrb = true
      end

      c.Debug("Event", "Shadow Word: Death cast",
              "a.SinceSWD < 9", a.SinceSWD < 9, "cd", swdcd,
              "=>", swdGivesOrb)

      a.Orbs = s.Power("player", SPELL_POWER_SHADOW_ORBS)
      if c.IsCasting("Mind Blast") then
         a.Orbs = min(a.maxOrbs, a.Orbs + 1)
      elseif c.IsCasting("Devouring Plague") then
         a.Orbs = max(0, a.Orbs - 3)
      elseif c.IsCasting("Shadow Word: Death")
         and s.HealthPercent() <= 20
         and a.swdGivesOrb()
      then
         a.Orbs = min(a.maxOrbs, a.Orbs + 1)
      end

      a.Surges = c.GetBuffStack("Surge of Darkness")
      if c.IsCasting("Mind Spike") then
         a.Surges = a.Surges - 1
      end

      a.InExecute = s.HealthPercent() <= 20

      c.FlashAll(
         "Power Word: Shield for Shadow",
         "Power Infusion",
         "Mindbender",
         "Shadowfiend",
         "Desperate Prayer",
         "Vampiric Embrace",
         "Dispel Magic",
         "Silence"
      )

      c.DelayPriorityFlash(
         "Devouring Plague",
         "Mind Blast",
         "Shadow Word: Death for Orb",
         "Insanity",
         -- "Mind Flay (Insanity) Delay",
         "Mind Spike under Surge of Darkness",
         "Shadow Word: Pain",
         "Vampiric Touch",
         "Shadow Word: Death without Orb",
         "Cascade",
         "Divine Star",
         "Halo",
         "Shadow Word: Pain Early",
         "Vampiric Touch Early",
         "Mind Blast Delay",
         "Shadow Word: Death Delay",
         "Mind Flay"
      )
   end,

   MovementFallthrough = function()
      c.PriorityFlash(
         "Shadow Word: Death",
         "Shadow Word: Pain Moving")
   end,

   FlashOutOfCombat = function()
      if not a.WarnedAboutRotation then
         a.WarnedAboutRotation = true
         print(
            "|cFFFF0000WARNING: The Shadow Priest rotation works, more or less, to level 90, but has not been updated to level 100.  It require major rework and testing to be working right.  This will happen, and is in progress.|r -- SlippyCheeze"
         )
      end

      c.FlashAll("Dispersion")
   end,

   FlashAlways = function()
      c.FlashAll(
         "Power Word: Fortitude",
         "Shadowform"
      )
   end,

   CastSucceeded = function(info)
      if c.InfoMatches(info, "Shadow Word: Death") then
         lastSWDCast = GetTime()
         lastSWDCD = c.GetCooldown("Shadow Word: Death")
         c.Debug("Event", "Shadow Word: Death cast",
                 "cast", lastSWDCast, "cd", lastSWDCD)
      end
   end,

   AuraApplied = function(spellID)
      if c.IdMatches(spellID, "Mind Flay", "Insanity") then
         a.FlayTick = c.GetHastedTime(1)
         c.Debug("Event", "Mind Flay ticks every", a.FlayTick)
      end
   end,

   ExtraDebugInfo = function()
      return string.format(
         "o:%d s:%d s:%d s:%.1f b:%.1f",
         a.Orbs, a.Surges, a.swdGivesOrb(), a.SinceSWD, c.GetBusyTime()
      )
   end,
}
